#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "odicon.h"
#include "lseparable.h"


static void test(int dim, double f(int,int*), int init1[], int init2[],
                 int limit1, int limit2, int limit3, int limit4, int limit5) {
    double ans1, ans2;
    int width = 20 * sqrt(dim);
    int x[1000];

    printf("--------------------------\n");
    printf("Wolfe\n");
    counter_reset();  submodular_type = FW;
    ans1 = sf_minimize(dim, f, init2, 1, x, -1);
    printf("f = %g, |S| = %d\n", ans1, x[0]);
    counter_print();

    if (dim > limit1) return;

    printf("--------------------------\n");
    printf("IFF\n");
    counter_reset();  submodular_type = IFF;
    ans2 = sf_minimize(dim, f, init2, 1, x, -1);
    printf("f = %g, |S| = %d\n", ans2, x[0]);
    counter_print();
    assert(fcmp(ans1,ans2) == 0);

    printf("--------------------------\n");
    printf("Wolfe + wide\n");
    counter_reset();  submodular_type = FW;
    ans1 = sf_minimize(dim, f, init2, width, x, -1);
    printf("f = %g, |S| = %d\n", ans1, x[0]);
    counter_print();

    printf("--------------------------\n");
    printf("IFF + wide\n");
    counter_reset();  submodular_type = IFF;
    ans2 = sf_minimize(dim, f, init2, width, x, -1);
    printf("f = %g, |S| = %d\n", ans2, x[0]);
    counter_print();
    assert(fcmp(ans1,ans2) == 0);

    if (dim > limit2) return;

    printf("--------------------------\n");
    printf("Wolfe + relax\n");
    counter_reset();  submodular_type = FW;
    ans1 = sf_minimize(dim, f, init1, 1, x, -1);
    printf("f = %g, |S| = %d\n", ans1, x[0]);
    counter_print();

    printf("--------------------------\n");
    printf("IFF + relax\n");
    counter_reset();  submodular_type = IFF;
    ans2 = sf_minimize(dim, f, init1, 1, x, -1);
    printf("f = %g, |S| = %d\n", ans2, x[0]);
    counter_print();
    assert(fcmp(ans1,ans2) == 0);

    printf("--------------------------\n");
    printf("Wolfe + wide + relax\n");
    counter_reset();  submodular_type = FW;
    ans1 = sf_minimize(dim, f, init1, sqrt(dim), x, -1);
    printf("f = %g, |S| = %d\n", ans1, x[0]);
    counter_print();

    if (dim > limit3) return;

    printf("--------------------------\n");
    printf("IFF + wide + relax\n");
    counter_reset();  submodular_type = IFF;
    ans2 = sf_minimize(dim, f, init1, sqrt(dim), x, -1);
    printf("f = %g, |S| = %d\n", ans2, x[0]);
    counter_print();
    assert(fcmp(ans1,ans2) == 0);

}

static struct lseparable_st *lseparable_inst;
static double ffunc(int dim, double x[]) { return lseparable(dim, x, lseparable_inst); }
/*static double ifunc(int dim, int x[]) { return int_func(dim, ffunc, x); }*/
static double ifunc(int dim, int x[]) { return intint_func(dim, ffunc, x); }

void test_submodular(const int dim) {
    int i;
    double *fans = fvector_new(dim, 190, -200, 200);
    int    *ians = ivector_new(dim, 190, -200, 200);
    int    *init = ivector_new(dim, 190, -200, 200); /* starting point */

    for (i=0; i<dim; i++) {
        fans[i] = init[i] = irandom(-10*dim, +10*dim);
    }
    /*
    {int data[] = {-19, -14, 33, -29};
    for (i=0; i<dim; i++) init[i] = data[i];}
    */

    lseparable_inst = lseparable_make(dim);
#ifdef DEBUG    
    lseparable_output(lseparable_inst);
#endif /* DEBUG */
    lbfgs_call(dim, ffunc, fans);
    for (i=0; i<dim; i++) ians[i] = (int)floor(0.5 + fans[i]);

    test(dim, ifunc, ians, init, 200, 120, 55, 0, 0);
}
